//
// c h i l d . c
//
// child process for squash signal
//
// Dec. 7, 2002 by W.Nishida (http://www.skyfree.org)
//

#include <stdio.h>	// printf(), fopen(), fprintf(), fscanf()
#include <stdlib.h>	// exit()
#include <unistd.h>	// alarm(), getpid(), sleep(), unlink()
#include <signal.h>	// signal(), kill(), SIG*

#define PPIDFILE	"parent.pid"	// Parent's PID filename
#define CPIDFILE	"child.pid"	// Child'd PID filename

int ppid, cpid;		// Parent's PID, Child's PID
int balls = 0;		// How many balls I received?
#define MAXBALLS 4

void sig_usr1(int sig) {
  printf("Received\n");
  balls++;				// Increment ball counter
  sleep(1);				// Wait for 1sec
  kill(ppid, SIGUSR1);			// Send next ball
  printf("Sent\n");
  if (balls == MAXBALLS)		// End of game?
    kill(cpid, SIGALRM);		// Then, terminate process
 }

void sig_alarm(int sig) {
  if (balls == MAXBALLS)
    printf("Game over!\n");
  else
    printf("Time is over!\n");
  unlink(CPIDFILE);
  exit(0);
 }

int main() {
  int ret;
  FILE *fp, *fc;

  signal(SIGINT,  sig_alarm);
  signal(SIGUSR1, sig_usr1);
  signal(SIGALRM, sig_alarm);

  // Make my PID number file

  cpid = getpid();			// Get my PID
  printf("This is child. My PID is %d\n", cpid);
  fp = fopen(CPIDFILE, "w");		// Open child PID file
  if (fp == NULL) {			// Open failure
    printf("Child file open failure!\n");
    return 1;
   }
  ret = fprintf(fp, "%d\n", cpid);	// Write down my PID
  if (ret < 0) {			// Write failure
    printf("Child file write failure!\n");
    return 1;
   }
  fclose(fp);				// Close file

  // Activate watchdog timer

  alarm(15);				// Wait for 15 seconds

  // Open parent's PID number file

  while (1) {
    fc = fopen(PPIDFILE, "r");		// Open parent PID file
    if (fc == NULL) {			// File does not exist
      sleep(1);				// Wait 1sec
      continue;				// Then, try again
     }
    break;				// Found!
   }
  ret = fscanf(fc, "%d", &ppid);	// Read parent's PID
  if (ret < 0) {			// Read failure
    printf("Parent PID file read failure!\n");
    return 1;
   }
  printf("Parent's PID is %d\n", ppid);
  fclose(fc);				// Close file

  // Start new game!

  printf("Let's start squashing!\n");
  while(1) {};				// Loop forever
 }
