//
// k e y d u m p . c
//
// Dump keyboard data in realtime
//
// Nov. 30, 2002 by W.Nishida (http://www.skyfree.org)
//

#include <stdio.h>	// printf()
#include <termios.h>	// termios{}, tcgetattr(), tcsetattr()
#include <unistd.h>	// STDIN_FILENO
#include <stdlib.h>	// exit()
#include <signal.h>	// signal(), SIGALRM
#include <unistd.h>	// alarm()
#include <sys/ioctl.h>	// ioctl()
#include <linux/kd.h>	// KDSKBMODE, K_RAW, K_MEDIUMRAW, K_XLATE
#include <string.h>	// strcmp()

struct termios tm, ntm;	// "termios" structure holders
int counter = 0;	// Counter for "blank" key state
int key_mode = 1;	// Selected keyboard mode
char* mode_name[] = {	// Name of each keyboard mode
  "Raw mode (system scan code)",
  "Translation mode (ASCII code)",
  "Medium raw mode (Linux scan code)"
 };

void sig_alarm(int sig) {
  int ret = 0;

  printf("\nI detected %d times of \"empty key\".\n", counter);

  // Restore the original "termios"

  ret = tcsetattr(STDIN_FILENO, TCSANOW, &tm);
  if (ret < 0) {
    printf("tcsetattr() failure!\n");
    ret = 1;
   }

  // Restore keyboard mode to "XLATE"

  if (key_mode != 1) {
  ret = ioctl(STDIN_FILENO, KDSKBMODE, K_XLATE);
    if (ret < 0) {
      printf("ioctl(KDSKBMODE) failure!\n");
      ret = 1;
     }
   }
  exit(ret);
 }

int setup_stdin(void) {
  int ret;

  // Change keyboard mode

  if (key_mode != 1) {
    ret = ioctl(STDIN_FILENO, KDSKBMODE, key_mode);
    if (ret < 0) {
      printf("ioctl(KDSKBMODE) failure!\n");
      printf("Sorry, you might activate me on X-window.\n");
      printf("Please call me in a /dev/tty console!\n");
      return 1;
     }
   }

  // Read current "termios" parameter

  ret  = tcgetattr(STDIN_FILENO, &tm);
  ret |= tcgetattr(STDIN_FILENO, &ntm);
  if (ret < 0) {
    printf("tcgetattr() failure!\n");
    return 1;
   }

  // Setup realtime key scanning

  ntm.c_iflag &= ~(ISTRIP | INLCR | ICRNL | IGNCR | IXON | IXOFF);
  ntm.c_lflag &= ~(ECHO | ICANON | ISIG);
  ntm.c_cc[ VTIME ] = 0;	// We need no wait
  ntm.c_cc[ VMIN  ] = 0;	// Blank data is acceptable
  ret = tcsetattr(STDIN_FILENO, TCSANOW, &ntm);
  if (ret < 0) {
    printf("tcsetattr() failure!\n");
    return 1;
   }
  return 0;
 }

int main(int argc, char** argv) {
  int ret;
  unsigned char key;

  if (argc == 2) {
    if (strcmp(argv[1], "ascii") == 0)
      key_mode = K_XLATE;
    else if (strcmp(argv[1], "raw") == 0)
      key_mode = K_RAW;
    else if (strcmp(argv[1], "linux") == 0)
      key_mode = K_MEDIUMRAW;
   }
  else if (argc > 2) {
    printf("Usage: realkey [ ascii | raw | linux ]\n");
    return 1;
   }

  ret = setup_stdin();
  if (ret)
    return 1;

  signal(SIGALRM, sig_alarm);		// Setup SIGALRM handler
  alarm(10);				// Activate 10 second alarm
  printf("Keyboard mode = %s\n", mode_name[ key_mode ]);
  printf("Hit any key please...\n");
  while (1) {
    ret = read(STDIN_FILENO, &key, 1);	// Try to read a data
    if (ret == 0) {			// There is no data...
      counter++;			// Then, increment counter
      continue;				// And scan again
     }
    printf("0x%02X : %3d\n", key, key);	// We got data!
   }
 }
