//
// p a r e n t . c
//
// parent process for squash signal
//
// Dec. 7, 2002 by W.Nishida (http://www.skyfree.org)
//

#include <stdio.h>	// printf(), fopen(), fprintf(), fscanf()
#include <stdlib.h>	// exit()
#include <unistd.h>	// alarm(), getpid(), sleep(), unlink()
#include <signal.h>	// signal(), kill(), SIG*

#define PPIDFILE	"parent.pid"	// Parent's PID filename
#define CPIDFILE	"child.pid"	// Child'd PID filename

int ppid, cpid;		// Parent's PID, Child's PID
int balls = 0;		// How many balls I received?
#define MAXBALLS 4

void sig_usr1(int sig) {
  printf("Received\n");
  balls++;				// Increment ball counter
  if (balls == MAXBALLS)		// End of game?
    kill(ppid, SIGALRM);		// Then, terminate process
  sleep(1);				// Wait for 1sec
  kill(cpid, SIGUSR1);			// Send next ball
  printf("Sent\n");
 }

void sig_alarm(int sig) {
  if (balls == MAXBALLS)
    printf("Game over!\n");
  else
    printf("Time is over!\n");
  unlink(PPIDFILE);
  exit(0);
 }

int main() {
  int ret;
  FILE *fp, *fc;

  signal(SIGINT,  sig_alarm);
  signal(SIGUSR1, sig_usr1);
  signal(SIGALRM, sig_alarm);

  // Make my PID number file

  ppid = getpid();			// Get my PID
  printf("This is parent. My PID is %d\n", ppid);
  fp = fopen(PPIDFILE, "w");		// Open parent PID file
  if (fp == NULL) {			// Open failure
    printf("Parent file open failure!\n");
    return 1;
   }
  ret = fprintf(fp, "%d\n", ppid);	// Write down my PID
  if (ret < 0) {			// Write failure
    printf("Parent file write failure!\n");
    return 1;
   }
  fclose(fp);				// Close file

  // Activate watchdog timer

  alarm(15);				// Wait for 15 seconds

  // Open child's PID number file

  printf("I'm waiting for child...\n");
  while (1) {
    fc = fopen(CPIDFILE, "r");		// Open child PID file
    if (fc == NULL) {			// File does not exist
      sleep(1);				// Wait 1sec
      continue;				// Then, try again
     }
    break;				// Found!
   }
  ret = fscanf(fc, "%d", &cpid);	// Read child's PID
  if (ret < 0) {			// Read failure
    printf("Child PID file read failure!\n");
    return 1;
   }
  printf("Child's PID is %d\n", cpid);
  fclose(fc);				// Close file

  // Start new game!

  printf("Let's start squashing!\n");
  kill(cpid, SIGUSR1);			// Send first ball to child
  printf("Sent\n");
  while(1) {};				// Loop forever
 }
